@@ -21,7 +21,7 @@ if [ -n "${ARCH:-}" ]; then
 fi
 buildArgs+=( validate "$SUITE" "@$epoch" )
 
-checkFile="validate/$serial/${ARCH:-amd64}/${CODENAME:-$SUITE}/rootfs.tar.xz"
+#checkFile="validate/$serial/${ARCH:-amd64}/${CODENAME:-$SUITE}/rootfs.tar.xz"
 mkdir -p validate
 
 set -x
@@ -29,5 +29,5 @@ set -x
 ./scripts/debuerreotype-version
 ./docker-run.sh --pull ./examples/debian.sh "${buildArgs[@]}"
 
-real="$(sha256sum "$checkFile" | cut -d' ' -f1)"
-[ -z "$SHA256" ] || [ "$SHA256" = "$real" ]
+#real="$(sha256sum "$checkFile" | cut -d' ' -f1)"
+#[ -z "$SHA256" ] || [ "$SHA256" = "$real" ]
